@@ -17,7 +17,6 @@ import re
 
 from django.conf import settings
 from django.contrib import auth
-from django.contrib.auth import middleware
 from django.contrib.auth import models
 from django.utils import timezone
 from keystoneauth1.identity import v2 as v2_auth
@@ -62,6 +61,10 @@ def get_user(request):
 
 
 def patch_middleware_get_user():
+    # NOTE(adriant): We can't import middleware until our customer user model
+    # is actually registered, otherwise a call to get_user_model within the
+    # middleware module will fail.
+    from django.contrib.auth import middleware
     middleware.get_user = middleware_get_user
     auth.get_user = get_user
 
@@ -186,6 +189,7 @@ def get_websso_url(request, auth_url, websso_auth):
     :type websso_auth: string
 
     Example of horizon WebSSO setting::
+
         WEBSSO_CHOICES = (
             ("credentials", "Keystone Credentials"),
             ("oidc", "OpenID Connect"),
@@ -207,12 +211,14 @@ def get_websso_url(request, auth_url, websso_auth):
     The value in WEBSSO_IDP_MAPPING is expected to be a tuple formatted as
     (<idp_id>, <protocol_id>). Using the values found, a IdP/protocol
     specific URL will be constructed:
+
         /auth/OS-FEDERATION/identity_providers/<idp_id>
         /protocols/<protocol_id>/websso
 
     If no value is found from the WEBSSO_IDP_MAPPING dictionary, it will
     treat the value as the global WebSSO protocol <protocol_id> and
     construct the WebSSO URL by:
+
         /auth/OS-FEDERATION/websso/<protocol_id>
 
     :returns: Keystone WebSSO endpoint.
@@ -369,7 +375,8 @@ def get_project_list(*args, **kwargs):
     return projects
 
 
-def default_services_region(service_catalog, request=None):
+def default_services_region(service_catalog, request=None,
+                            selected_region=None):
     """Returns the first endpoint region for first non-identity service.
 
     Extracted from the service catalog.
@@ -393,8 +400,7 @@ def default_services_region(service_catalog, request=None):
                 LOG.error('No regions can be found in the service catalog.')
                 return None
 
-        selected_region = None
-        if request:
+        if request and selected_region is None:
             selected_region = request.COOKIES.get('services_region',
                                                   available_regions[0])
         if selected_region not in available_regions:
@@ -436,12 +442,14 @@ def using_cookie_backed_sessions():
 def get_admin_roles():
     """Common function for getting the admin roles from settings
 
-       Returns:
-        Set object including all admin roles.
-        If there is no role, this will return empty.
+    :return:
+      Set object including all admin roles.
+      If there is no role, this will return empty::
+
         {
             "foo", "bar", "admin"
         }
+
     """
     admin_roles = {role.lower() for role
                    in getattr(settings, 'OPENSTACK_KEYSTONE_ADMIN_ROLES',
@@ -454,9 +462,10 @@ def get_role_permission(role):
 
     This format is 'openstack.roles.xxx' and 'xxx' is a real role name.
 
-    Returns:
+    :returns:
         String like "openstack.roles.admin"
         If role is None, this will return None.
+
     """
     return "openstack.roles.%s" % role.lower()
 
@@ -466,14 +475,16 @@ def get_admin_permissions():
 
     This format is 'openstack.roles.xxx' and 'xxx' is a real role name.
 
-    Returns:
-        Set object including all admin permission.
-        If there is no permission, this will return empty.
+    :returns:
+       Set object including all admin permission.
+       If there is no permission, this will return empty::
+
         {
             "openstack.roles.foo",
             "openstack.roles.bar",
             "openstack.roles.admin"
         }
+
     """
     return {get_role_permission(role) for role in get_admin_roles()}
 
